package edu.uci.ics.asterix.om.base;

import java.security.SecureRandom;
import java.util.UUID;

import org.json.JSONException;
import org.json.JSONObject;

import edu.uci.ics.asterix.common.exceptions.AsterixException;
import edu.uci.ics.asterix.om.types.BuiltinType;
import edu.uci.ics.asterix.om.types.IAType;
import edu.uci.ics.asterix.om.visitors.IOMVisitor;

public class AUUID implements IAObject {

    protected static class Holder {
        static final SecureRandom srnd = new SecureRandom();
    }

    protected long msb;
    protected long lsb;

    public AUUID(UUID uuid) {
        msb = uuid.getMostSignificantBits();
        lsb = uuid.getLeastSignificantBits();
    }

    public AUUID(long msb, long lsb) {
        this.msb = msb;
        this.lsb = lsb;
    }

    public long getMostSignificantBits() {
        return msb;
    }

    public long getLeastSignificantBits() {
        return lsb;
    }

    public static AUUID randomUUID() {
        long[] bits = new long[2];
        byte[] randomBytes = new byte[16];
        Holder.srnd.nextBytes(randomBytes);
        uuidBitsFromBytes(bits, randomBytes);
        return new AUUID(bits[0], bits[1]);
    }

    protected static void uuidBitsFromBytes(long[] bits, byte[] randomBytes) {
        bits[0] = 0;
        bits[1] = 0;
        randomBytes[6] &= 0x0f; /* clear version        */
        randomBytes[6] |= 0x40; /* set to version 4     */
        randomBytes[8] &= 0x3f; /* clear variant        */
        randomBytes[8] |= 0x80; /* set to IETF variant  */
        for (int i = 0; i < 8; ++i) {
            bits[0] = (bits[0] << 8) | (randomBytes[i] & 0xff);
        }
        for (int i = 8; i < 16; ++i) {
            bits[1] = (bits[1] << 8) | (randomBytes[i] & 0xff);
        }
    }

    @Override
    public JSONObject toJSON() throws JSONException {
        JSONObject json = new JSONObject();
        json.put("AUUID", toString());
        return json;
    }

    @Override
    public IAType getType() {
        return BuiltinType.AUUID;
    }

    @Override
    public void accept(IOMVisitor visitor) throws AsterixException {
        visitor.visitAUUID(this);
    }

    @Override
    public boolean deepEqual(IAObject obj) {
        if (!(obj instanceof AUUID)) {
            return false;
        }
        AUUID oUUID = (AUUID) obj;
        return oUUID.msb == this.msb && oUUID.lsb == this.lsb;
    }

    @Override
    public int hash() {
        long hilo = msb ^ lsb;
        return ((int) (hilo >> 32)) ^ (int) hilo;
    }

    @Override
    public String toString() {
        return "AUUID: {"
                + (digits(msb >> 32, 8) + "-" + digits(msb >> 16, 4) + "-" + digits(msb, 4) + "-"
                        + digits(lsb >> 48, 4) + "-" + digits(lsb, 12)) + "}";
    }

    private static String digits(long val, int digits) {
        long hi = 1L << (digits * 4);
        return Long.toHexString(hi | (val & (hi - 1))).substring(1);
    }
}
